-------------------------------------------------------------------------------
-- MAC developpement, no license!
-- Targeted on xilinx signal description to di IPBUS as proposed by CERN
-- Core is full duplex only (no collision management)
-- No pause management (clause 31b, page 760 section 2)
-- Pause frames passed to user
-- use minimum legal Interframe Gap Delay
--
-- Destination address (DA), Source Address (SA) and Length (LT) are provided
-- by user interface via the data port
-- O. Bourrion 12/2013
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity tri_mode_eth_mac is
port (
	glbl_rstn : in std_logic;
	rx_axi_rstn : in std_logic;
	tx_axi_rstn : in std_logic;
	rx_axi_clk : in std_logic;
	rx_reset_out : out std_logic;
	rx_axis_mac_tdata : out std_logic_vector(7 downto 0);
	rx_axis_mac_tvalid : out std_logic;
	rx_axis_mac_tlast : out std_logic;
	rx_axis_mac_tuser : out std_logic;
	rx_statistics_vector : out std_logic_vector(27 downto 0);
	rx_statistics_valid : out std_logic;
	tx_axi_clk : in std_logic;
	tx_reset_out : out std_logic;
	tx_axis_mac_tdata : in std_logic_vector(7 downto 0);
	tx_axis_mac_tvalid : in std_logic;
	tx_axis_mac_tlast : in std_logic;
	tx_axis_mac_tuser : in std_logic;
	tx_axis_mac_tready : out std_logic;
	tx_ifg_delay : in std_logic_vector(7 downto 0);--! Intrame gap delay extension-> unused
	tx_statistics_vector : out std_logic_vector(31 downto 0);
	tx_statistics_valid : out std_logic;
	pause_req : in std_logic;
	pause_val : in std_logic_vector(15 downto 0);
	speed_is_100 : out std_logic;
	speed_is_10_100 : out std_logic;
	gmii_txd : out std_logic_vector(7 downto 0);
	gmii_tx_en : out std_logic;
	gmii_tx_er : out std_logic;
	gmii_rxd : in std_logic_vector(7 downto 0);
	gmii_rx_dv : in std_logic;
	gmii_rx_er : in std_logic;
	rx_mac_config_vector : in std_logic_vector(79 downto 0);
	tx_mac_config_vector : in std_logic_vector(79 downto 0)
	);
end tri_mode_eth_mac;

architecture rtl of tri_mode_eth_mac is
component mac_rx_engine
port (
	rstn : in std_logic;
	clk : in std_logic;
	
	rx_tdata : out std_logic_vector(7 downto 0);
	rx_tvalid : out std_logic;
	rx_tlast : out std_logic;
	rx_tuser : out std_logic;
	rx_statistics_vector : out std_logic_vector(27 downto 0);
	rx_statistics_valid : out std_logic;
	rx_mac_config_vector : in std_logic_vector(79 downto 0);
	
	gmii_rxd : in std_logic_vector(7 downto 0);
	gmii_rx_dv : in std_logic;
	gmii_rx_er : in std_logic
	);
end component;

component mac_tx_engine
port (
	rstn : in std_logic;
	
	-- to user interface
	clk : in std_logic;
	tx_tdata : in std_logic_vector(7 downto 0); --!frame data to be Xfrerd
	tx_tvalid : in std_logic; --! TX data valid
	tx_tlast : in std_logic; --! final Xfer of frame
	tx_tuser : in std_logic; --! Signal tx error
	tx_tready : out std_logic; --! Transmitter ready, qualifies when data is taken by MAC
	tx_statistics_vector : out std_logic_vector(31 downto 0); 
	tx_statistics_valid : out std_logic;
	tx_mac_config_vector : in std_logic_vector(79 downto 0);

	-- to gmii
	gmii_txd : out std_logic_vector(7 downto 0);
	gmii_tx_en : out std_logic;
	gmii_tx_er : out std_logic
	);
end component;

begin
speed_is_100<='0';
speed_is_10_100<='0';
rx_reset_out<='0';
tx_reset_out<='0';

lbl_mac_rx :  mac_rx_engine port map(
	rstn=>glbl_rstn,
	clk=>rx_axi_clk,
	
	rx_tdata=>rx_axis_mac_tdata,
	rx_tvalid=>rx_axis_mac_tvalid,
	rx_tlast=>rx_axis_mac_tlast,
	rx_tuser=>rx_axis_mac_tuser,
	rx_statistics_vector=>rx_statistics_vector,
	rx_statistics_valid=>rx_statistics_valid,
	rx_mac_config_vector=>rx_mac_config_vector,
	
	gmii_rxd=>gmii_rxd,
	gmii_rx_dv=>gmii_rx_dv,
	gmii_rx_er=>gmii_rx_er
	);

lbl_mac_tx : mac_tx_engine port map(
	rstn=>glbl_rstn,
	
	-- to user interface
	clk=>tx_axi_clk,
	tx_tdata=>tx_axis_mac_tdata,
	tx_tvalid=>tx_axis_mac_tvalid,
	tx_tlast=>tx_axis_mac_tlast,
	tx_tuser=>tx_axis_mac_tuser,
	tx_tready=>tx_axis_mac_tready,
	tx_statistics_vector=>tx_statistics_vector,
	tx_statistics_valid=>tx_statistics_valid,
	tx_mac_config_vector=>tx_mac_config_vector,

	-- to gmii
	gmii_txd=>gmii_txd,
	gmii_tx_en=>gmii_tx_en,
	gmii_tx_er=>gmii_tx_er
	);

end rtl;
