-------------------------------------------------------------------------------
-- Pulse stretcher
-- use to lengthen a pulse so that it can be sampled correctly by an other 
-- (faster clock)
--
-- O. Bourrion July 2015
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use IEEE.NUMERIC_STD.ALL;

entity ntp_pulse_stretcher is
generic(
	stretch_val : natural
	);
port(
	clk : in std_logic;
	rst : in std_logic;
	
	pulse_in : in std_logic;
	pulse_out : out std_logic
	);
end ntp_pulse_stretcher;

architecture RTL of ntp_pulse_stretcher is
signal stretch_cnt : natural range 0 to stretch_val-1;
type def_pulse_state is (idle,pulse);
signal pulse_state : def_pulse_state;

begin
process(clk)
begin
	if rising_edge(clk) then
		if rst='1' then
			pulse_out<='0';
			pulse_state<=idle;
		else
			case pulse_state is
				when idle =>
					stretch_cnt<=0;
					if pulse_in='1' then
						pulse_out<='1';
						pulse_state<=pulse;
					end if;

				when pulse =>
					if stretch_cnt=stretch_val-1 then
						pulse_out<='0';
						pulse_state<=idle;
					else
						stretch_cnt<=stretch_cnt+1;
					end if;
			end case;
		end if;
	end if;
end process;
end RTL;
