library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

use work.pack_ipv4_types.all;

package pack_simu_udp is
type ipb_packet_typ is array (natural range <>) of bit_vector(31 downto 0);
-------------------------------------------------------------------------------
-- this type expect packet size in word32 at add 0
-- following words must be a valid ipbus packet
-------------------------------------------------------------------------------


procedure send_udp_buffer(
	constant DST_PORT		: in std_logic_vector(15 downto 0);
	constant SRC_PORT		: in std_logic_vector(15 downto 0);
	constant DST_IP_ADD		: in std_logic_vector(31 downto 0);
	variable is_LE			: in boolean;
	constant data_buf		: in ipb_packet_typ;

	signal clk125					: in std_logic;
	signal udp_tx_data_out_ready	: in std_logic;
	signal udp_tx_start				: out std_logic;
	signal udp_txi					: out udp_tx_type
);

end package pack_simu_udp;

package body pack_simu_udp is

procedure send_udp_buffer(
	constant DST_PORT		: in std_logic_vector(15 downto 0);
	constant SRC_PORT		: in std_logic_vector(15 downto 0);
	constant DST_IP_ADD		: in std_logic_vector(31 downto 0);
	variable is_LE			: in boolean;
	constant data_buf		: in ipb_packet_typ;

	signal clk125					: in std_logic;
	signal udp_tx_data_out_ready	: in std_logic;
	signal udp_tx_start				: out std_logic;
	signal udp_txi					: out udp_tx_type
) is
variable txsize : natural;
variable byte2use : natural;
begin
	txsize:=conv_integer(to_stdlogicvector(data_buf(0)));
	udp_tx_start<='0';
	
	udp_txi.hdr.dst_port<=DST_PORT;
	udp_txi.hdr.src_port<=SRC_PORT;
	udp_txi.hdr.data_length<=conv_std_logic_vector(txsize*4,16);
	udp_txi.hdr.dst_ip_addr<=DST_IP_ADD;
	udp_txi.hdr.checksum<=(others=>'0');

	wait until rising_edge(clk125);
	udp_tx_start<='1';
	udp_txi.data.data_out_valid<='1';

	for i in 1 to txsize loop
		for byte in 0 to 3 loop
			if is_LE then
				byte2use:=byte;
			else
				byte2use:=3-byte;
			end if;
			udp_txi.data.data_out<=to_stdlogicvector(data_buf(i)(8*(byte2use+1)-1 downto 8*byte2use));

			if i=txsize and byte=3 then
				udp_txi.data.data_out_last<='1';
				udp_tx_start<='0';
			end if;

			while true loop
				wait until rising_edge(clk125);
				if udp_tx_data_out_ready='1' then
					exit;
				end if;
			end loop;
		end loop;
	end loop;

	udp_txi.data.data_out_last<='0';
	udp_txi.data.data_out_valid<='0';
	wait until rising_edge(clk125);

end send_udp_buffer;

end package body pack_simu_udp;

